spiral.heatmap <- function(period, season, x, 
              palette=blues9, bin.type="q",
              na.col="yellow", start=0, 
              clockwise=FALSE, eye.rad = 0.5, 
              bg.col = par("bg"), narc=21, 
              labels=NULL, label.pos=NULL, 
              label.col=par("fg"), label.cex=0.8,
              legend=TRUE, legend.title="Legend",
              legend.col=par("fg"), legend.cex=0.8, 
              legend.labels = NULL, legend.pos="r",
              xlim=c(-1.3,ifelse(legend && legend.pos=="r", 2, 1.3)),
              ylim=c(ifelse(legend && legend.pos=="b", -2, -1.3), 1.3), 
              store=FALSE, ...){
  ## if season is a factor, its levels are used as default labels
  ## total radius is normalized to 1
  stopifnot(length(period)==length(season))
  stopifnot(length(period)==length(x))
  stopifnot(bin.type %in% c("q","e"))
  tvals <- unique(period)
  ## season should not be character!
  if (is.character(season)) warning("season should be numeric or factor,\nas character sorting can yield unexpected results")
  ## if season is a factor, sorted in the order of levels
  svals <- sort(unique(season))
  if (!is.factor(season)) 
    season <- factor(season, levels=svals)
  ## incomplete check for correct ordering
  if (!all(sort(tvals)==tvals) )
    stop("period values must be sorted")
  ## incomplete check for completeness and uniqueness
  tabt <- table(period)
  tabtinner <- tabt[-c(1,length(tabt))]
  if (!all(tabtinner==length(svals))) 
    stop("there must be a unique x value for each combination of period and season")
  dat <- data.frame(period=period, season=season, x=x)
  s <- length(svals)
  nperiods <- length(tabt)
  ## assign season angle to dat
  if (clockwise) dat$angle <- (s+1 - as.numeric(dat$season))*2*pi/s + start*pi/180
    else dat$angle <- (as.numeric(dat$season)-1)*2*pi/s + start*pi/180
  dat$angle[dat$angle>2*pi] <- dat$angle[dat$angle>2*pi]-2*pi
  if (clockwise){
    dat$anglefrom <- dat$angle + pi/s
    dat$angleto <- dat$angle - pi/s
  } else{
    dat$anglefrom <- dat$angle - pi/s
    dat$angleto <- dat$angle + pi/s
  }
  ## there is a strip of width approx. (1-eye.rad)/nperiods
  npolys <- length(season)
  thickpolys <- (1-eye.rad)/(nperiods+1)
  slopecircrect <- seq(0, thickpolys/s, 
                       length.out = narc)
  radstart <- radmin <- eye.rad 
  if (is.factor(x)) {
    if(length(levels(x))>length(palette)) stop("too few colours in palette")
    dat$facx <- dat$x
    if (bin.type=="e" && is.null(legend.labels)) stop('With factor.valued x, use bin.type="q" \nor provide scale ends with legend.labels.')
  }
  else {
    if (bin.type=="q") dat$facx <- cut(dat$x, 
     breaks=quantile(dat$x, seq(0, 1,
                    length.out = length(palette) + 1), 
                    na.rm = TRUE), 
     include.lowest = TRUE, ...)
    else{
      legend.labels <- range(dat$x, na.rm=TRUE)
      dat$facx <- cut(dat$x, 
          breaks=length(palette), 
          include.lowest = TRUE)
    }
  }

  ## initialize plot
  par(mar=c(1,1,1,1))
  symbols(0, 0, xlab="", ylab="",
          circles=1, 
          inches=FALSE, asp=1, bg=bg.col, fg=bg.col, 
          xlim=xlim, ylim=ylim, 
          xaxt="n",yaxt="n",bty="n")
  if (store) ausdat <- data.frame(period=dat$period, 
                                  season=dat$season,
                                  fromx=NA, fromy=NA, 
                                  midx=NA, midy=NA, 
                                  tox=NA, toy=NA)
  for (i in 1:nrow(dat)){
    zeile <- dat[i,] 
    arc <- seq(zeile$anglefrom, zeile$angleto, 
               length.out = narc)
    from <- radstart + slopecircrect
    to <- from + thickpolys
    if (store) {
      ausdat[i,]$fromx <- (from[1]+to[1])/2*cos(arc[1])
      ausdat[i,]$fromy <- (from[1]+to[1])/2*sin(arc[1])
      midpos <- narc%/%2
      ausdat[i,]$midx <- (from[midpos]+to[midpos])/2*cos(arc[midpos])
      ausdat[i,]$midy <- (from[midpos]+to[midpos])/2*sin(arc[midpos])
      ausdat[i,]$tox <- (from[narc]+to[narc])/2*cos(arc[narc])
      ausdat[i,]$toy <- (from[narc]+to[narc])/2*sin(arc[narc])
    }
    radstart <- radstart + max(slopecircrect)
    if (!is.na(zeile$facx)) 
      polygon(c(from*cos(arc), rev(to)*cos(rev(arc))), 
            c(from*sin(arc), rev(to)*sin(rev(arc))),
            col=palette[as.numeric(zeile$facx)], 
            border = palette[as.numeric(zeile$facx)])
    else polygon(c(from*cos(arc), rev(to)*cos(rev(arc))), 
                 c(from*sin(arc), rev(to)*sin(rev(arc))),
                 col=na.col, 
                 border = na.col)
  }    

  aus <- list(period.range = range(period),
              rad.range = c(radmin, max(to)))
  
if (is.null(labels)) {
  ## prepare default labels
  labels <- as.character(dat[1:s,]$season)
  ## must be in degrees, like label.pos from call
  label.pos <- 180*dat[1:s,]$angle/pi
}
## add labels
  if (!is.null(labels)){
    stopifnot(length(labels)==length(label.pos))
    stopifnot(is.character(labels) || 
                is.numeric(labels) || 
                is.expression(labels))
  if (length(labels)>0){
      ## bring label.pos to radians
      label.pos <- label.pos/180*pi
    for (i in 1:length(labels)){
      lp <- label.pos[i]
      lab <- labels[i]
      ## is 1.05 a suitable position for labels?
      lx <- 1.05*cos(lp)
      ly <- 1.05*sin(lp)
      ## y adjustment
      if (lp < pi) a2 <- 0 else a2 <- 1
      ## text rotation and x adjustment dependent on lp
      if (lp < pi/2 || lp > 3*pi/2)
        text(lx, ly, adj=c(0, a2),
             lab, xpd=NA, 
             srt=180*lp/pi, col=label.col, 
             cex=label.cex, ...)
      else
        text(lx, ly,
             adj=c(1, a2),
             lab, xpd=NA, 
             srt=180*lp/pi+180, col=label.col, 
             cex=label.cex, ...)
    }
    }
    }
  if (legend) {
    ## logic: assume square region + legend
    ## for legend on the right, use ylim[2]+offset
    ## for legend at the bottom, use xlim[2]+offset
    if (bin.type=="q") legend.pos <- "r"
    if (legend.pos == "r"){
      xl <- ylim[2]+(xlim[2]-ylim[2])/4
      xr <- ylim[2]+(xlim[2]-ylim[2])/2
      yb <- -1; yt <- 1
    } else
    {
      yb <- xlim[1]+(ylim[1]-xlim[1])/4
      yt <- xlim[1]+(ylim[1]-xlim[1])/2
      xl <- -1; xr <- 1
    } 
    if (bin.type=="q"){
       color.legend(xl,yb,xr,yt,levels(dat$facx),palette,
                 align="rb", gradient="y", cex=legend.cex, 
                 col=legend.col, ...)
      text((xlim[2]+ylim[2])/2, 1, 
           legend.title, pos=3, 
           cex=legend.cex, col=legend.col)
    }
    else{
      if (legend.pos=="r"){
        color.legend(xl,yb,xr,yt,"",palette,
                   align="rb", gradient="y", cex=legend.cex, 
                   col=legend.col, ...)
        text((xl+xr)/2, -1, legend.labels[1],pos=1,xpd=NA, cex=legend.cex, col=legend.col)
        text((xl+xr)/2, 1, legend.labels[2],pos=3,xpd=NA, cex=legend.cex, col=legend.col)
        text((xl+xr)/2, min(1.1, ylim[2]), 
           legend.title, pos=3, 
           cex=legend.cex, col=legend.col)
      }
      else{
        color.legend(xl,yb,xr,yt,"",palette,
                     align="rb", gradient="x", cex=legend.cex, 
                     col=legend.col, ...)
        text(-1,(yb+yt)/2,legend.labels[1],pos=2,xpd=NA, cex=legend.cex, col=legend.col)
        text(1,(yb+yt)/2,legend.labels[2],pos=4,xpd=NA, cex=legend.cex, col=legend.col)
        text(0, yt, legend.title, pos=1,xpd=NA, cex=legend.cex, col=legend.col)
      }
    }
  }
  ## finalize returned object
  hilf <- dat$angle[1:s]*180/pi
  names(hilf) <- as.character(dat$season[1:s])
  aus$angles <- hilf[sort(as.numeric(dat[1:s,]$season), index.return=TRUE)$ix]
  hilf <- palette; names(hilf) <- levels(dat$facx)
  aus$heatcolors <- hilf
  if (is.numeric(x)) aus$xrange <- range(x, na.rm=TRUE)
  if (store) aus$points.xy <- ausdat
  invisible(aus)
}
